@@ -1,4 +1,4 @@
-/* $Id: httpd.c,v 1.19 2005/12/11 20:23:46 ichiro Exp $ */
+/* $Id: httpd.c,v 1.27 2005/12/12 05:18:34 ichiro Exp $ */
 /*
  * Copyright (c) 2004
  *	Ichiro FUKUHARA <ichiro@ichiro.org>.
@@ -52,7 +52,7 @@
 #include "httpd.h"
 
 void httpd_proc(struct tparam *, int);
-void print_command_page(int);
+void print_command_page(struct tparam *, int);
 
 gpointer
 httpd(gpointer param)
@@ -264,25 +264,40 @@ void httpd_proc(struct tparam *th2, int socket)
 			memset(cmd_cmd, 0, sizeof(cmd_cmd));    
 			memset(cmd_topic, 0, sizeof(cmd_topic));
 
+			if (post_length < 22) {
+				/* clear trans. empty flag */
+				th2->t_empty = 0;
+                                th2->t_lock = 0;
+                                goto index_start;
+			}
+
 			sprintf(post_message,	/* XXX temporary used */
 				"%.*s", post_length, s);
 
+			/* cname=&cname2=&cmd=part&topic= */
 			q = strtok(post_message, "&");
-				if (q != NULL) strcpy(cmd_cname, q + 6);
-					/* 6: cname= */
-			if (strstr(buf, "cmd=")) {
-				q = strtok(NULL, "&");
-				if (q != NULL) {
-					strcpy(cmd_cmd, q + 4);
-					/* 4: cmd= */
-				}
-			}
+			if (q != NULL) strcpy(cmd_cname, q + 6);	/* 6: cname= */
+
 			q = strtok(NULL, "&");
 			if (q != NULL) {
-				strcpy(cmd_buf, q + 5);
-					/* 5: toto= */
+				if (strlen(cmd_cname) == 0)
+					strcpy(cmd_cname, q + 7);	/* 7: cname2= */
 			}
 
+			q = strtok(NULL, "&");
+			if (q != NULL) {
+				strcpy(cmd_cmd, q + 4);		/* 4: cmd= */
+			}
+
+			q = strtok(NULL, "&");
+			if (q != NULL) {
+				strcpy(cmd_buf, q + 6); /* 6: topic= */
+			}
+#ifdef DEBUG
+			printf("post_message=%s\n", post_message);
+			printf("cname=%s\ncmd=%s\ntopic=%s\n",
+				cmd_cname, cmd_cmd, cmd_buf);
+#endif
 			/* make command message */
 			if (strcmp(cmd_cmd, "join") == 0) {
 				url_decode_rawdata(channel, cmd_cname);
@@ -352,7 +367,7 @@ command_start:
 		send_msg(socket, http_header1);
 		send_msg(socket, http_header2);
 
-		print_command_page(socket);
+		print_command_page(th2, socket);
 
 		send_msg(socket, http_footer);
 
@@ -477,8 +492,11 @@ int send_msg(int fd, char *msg)
 	return n;
 }
 
-void print_command_page(int socket)
+void print_command_page(struct tparam *th2, int socket)
 {
+	char str[C_NAME];
+	int i;
+
 	send_msg(socket,
 	   "<form action=\"/cmd\" method=\"post\">");
 	send_msg(socket,
@@ -486,16 +504,31 @@ void print_command_page(int socket)
 		accesskey=\"1\" value=\"OK[1]\">");
 	send_msg(socket,
 	   "<a accesskey=\"8\" href=\"/\"> back[8]</a><hr>");
+	
+	send_msg(socket, "Channel:");
 	send_msg(socket,
-	   "channel <input type=text name=\"cname\" size=\"10\">");
+	   "<br><select name=\"cname\">");
 	send_msg(socket,
-	   "<br><input type=radio name=\"cmd\" value=\"join\">JOIN");
+	   "<option value=\"\">specific</option>");
+	for (i = 0; i < th2->ch_count + 1; i++) {
+		sprintf(str, "<option value=\"%s\">%s</option>",
+			th2->name[i], th2->name[i]);
+		send_msg(socket, str);
+	}
+	send_msg(socket, "</select>");
+
+	send_msg(socket,
+	   "<br><input type=text name=\"cname2\" size=\"17\">");
+
 	send_msg(socket,
 	   "<br><input type=radio name=\"cmd\" value=\"part\">PART");
 	send_msg(socket,
-	   "<br><input type=radio name=\"cmd\" value=\"topic\">TOPIC");
+	   "<input type=radio name=\"cmd\" value=\"join\">JOIN");
+
 	send_msg(socket,
-	   "<br><input type=radio name=\"cmd\" value=\"nick\">NICK");
+	   "<hr><input type=radio name=\"cmd\" value=\"topic\">TOPIC");
 	send_msg(socket,
-	   "<input type=text name=\"toto\" size=\"7\"></form>");
+	   "<input type=radio name=\"cmd\" value=\"nick\">NICK");
+	send_msg(socket,
+	   "<br><input type=text name=\"topic\" size=\"17\"></form>");
 }
