@@ -1,4 +1,4 @@
-/* $Id: keitai.c,v 1.30 2005/12/11 21:40:01 ichiro Exp $ */
+/* $Id: keitai.c,v 1.32 2005/12/12 03:45:30 ichiro Exp $ */
 /*
  * Copyright (c) 2004
  *	Ichiro FUKUHARA <ichiro@ichiro.org>.
@@ -51,6 +51,7 @@ int
 keitai_start(struct tparam *th)
 {
 	char channel[C_NAME];
+	char id[IRC_MAX];
 
 #ifdef DEBUG
 		printf("%s\r\n", th->l_buff);
@@ -69,14 +70,18 @@ keitai_start(struct tparam *th)
 
 	/* detect JOIN channel */
 	if (strstr(th->l_buff, "JOIN")) {
-		sscanf(th->l_buff, ":%*s JOIN :%[^,]", channel);
-		if (channel[0] != '#') { /* for madoka4.2 */
-			sscanf(th->l_buff, ":%*s JOIN %[^,]", channel);
-#ifdef DEBUG
-			printf("JOIN: madoka hack\n");
-#endif
+		if (sscanf(th->l_buff, ":%[^!]!%*s JOIN :%[^,]",
+			   id, channel) == 2);
+		else if (sscanf(th->l_buff, ":%[^!]!%*s JOIN %[^,]",
+			        id, channel) == 2);
+		else if (sscanf(th->l_buff, ":%s JOIN %[^,]",
+				id, channel) == 2);
+
+		if (strcmp(th->real_nickname, id) == 0) {
+			join_recv(th, channel);
+		} else {
+			send_names(th, channel);
 		}
-		join_recv(th, channel);
 	}
 
 	if (strstr(th->l_buff, "PART"))
@@ -294,13 +299,18 @@ int part_recv(struct tparam *th)
 	memset(channel2, 0, sizeof(channel2));
 
 	/* get PART */
-	sscanf(th->l_buff, ":%s PART %s :%*[^,]",
-		id, channel);
-
-	/* send NAMES command */
-	if (strcmp(th->real_nickname, id) != 0) {
-		send_names(th, channel);
-		return (0);
+	if (sscanf(th->l_buff, ":%[^!]!%*s PART %s :%*[^,]",
+		id, channel) == 2) {
+		if (strcmp(th->real_nickname, id) != 0) {
+		   /* send NAMES command */
+		   send_names(th, channel);
+		   return (0);
+		}
+	} else if (sscanf(th->l_buff, ":%s PART %s :%*[^,]",
+		id, channel) == 2) {
+		if (strcmp(th->real_nickname, id) != 0) {
+			return (0);
+		}
 	}
 
 	/* channel;  #hoge:*.jp -> %hoge */
@@ -440,7 +450,7 @@ int send_names(struct tparam *th, char *channel)
 {
 	/* wait for flags clear */
 	while ((!th->t_empty) || (th->t_lock)) {
-		usleep(10000); /* 10ms */
+		usleep(100000); /* 10ms */
 	}
 	th->t_lock = 1;	/* now writing */
 
